from .return_class import AbstractApiClass


class LanguageDetectionPrediction(AbstractApiClass):
    """

    """

    def __init__(self, client, ):
        super().__init__(client, None)

    def __repr__(self):
        return f"LanguageDetectionPrediction()"

    def to_dict(self):
        return {}
